<?php
/*
*
* JetBackupCpanelPHPClient @ package
* Created By Idan Ben-Ezra
*
**/

class WHMCS {

	const HTTP_CLIENT_CURL = 1;
	const HTTP_CLIENT_FOPEN = 2;

	const DEFAULT_PROTOCOL = 'https';
	const DEFAULT_HTTP_CLIENT = self::HTTP_CLIENT_CURL;
	const DEFAULT_DEBUG = false;

	private $_data;

	/**
	 * JetBackup constructor.
	 *
	 * @param string $url
	 * @param string $identifier
	 * @param string $secret
	 *
	 * @throws Exception
	 */
	public function __construct($url='', $identifier='', $secret='') {
		$this->setURL($url);
		$this->setIdentifier($identifier);
		$this->setSecret($secret);
	}

	/**
	 * set the host you want to connect with
	 *
	 * @param $hostname
	 */
	public function setURL($url) { $this->_set('url', $url); }

	/**
	 * get the host to connect with
	 *
	 * @return mixed
	 */
	public function getURL() { return $this->_get('url'); }

	/**
	 * set the identifier to be authenticate with
	 *
	 * @param $identifier
	 */
	public function setIdentifier($identifier) { $this->_set('identifier', $identifier); }

	/**
	 * get the identifier to be authenticate with
	 *
	 * @return mixed
	 */
	public function getIdentifier() { return $this->_get('identifier'); }

	/**
	 * @param $secret
	 *
	 * @throws Exception
	 */
	public function setSecret($secret) { $this->_set('secret', $secret); }

	/**
	 * @return null
	 */
	public function getSecret() { return $this->_get('secret'); }

	/**
	 * @param $protocol
	 *
	 * @throws Exception
	 */
	public function setProtocol($protocol) {
		if ($protocol != 'http' && $protocol != 'https') throw new Exception('Invalid protocol provided');
		$this->_set('protocol', $protocol);
	}

	/**
	 * @return null
	 */
	public function getProtocol() { return $this->_get('protocol', self::DEFAULT_PROTOCOL); }

	/**
	 * @param $debug
	 */
	public function setDebug($debug) { $this->_set('debug', !!$debug); }

	/**
	 * @return null
	 */
	public function getDebug() { return $this->_get('debug', self::DEFAULT_DEBUG); }

	/**
	 * @param $http_client
	 *
	 * @throws Exception
	 */
	public function setHTTPClient($http_client) {
		if ($http_client != self::HTTP_CLIENT_CURL && $http_client != self::HTTP_CLIENT_FOPEN) throw new Exception('Invalid http client provided');
		$this->_set('http_client', $http_client);
	}

	/**
	 * @return null
	 */
	public function getHTTPClient() { return $this->_get('http_client', self::DEFAULT_HTTP_CLIENT); }

	/**
	 * @param $key
	 * @param $value
	 */
	private function _set($key, $value) { $this->_data[$key] = $value; }

	/**
	 * @param $key
	 * @param null $default
	 *
	 * @return null
	 */
	private function _get($key, $default=null) { return isset($this->_data[$key]) ? $this->_data[$key] : $default; }

	/**
	 * @throws Exception
	 */
	private function _validate() {
		if (!$this->getURL()) throw new Exception('No URL provided');
		if (!$this->getIdentifier()) throw new Exception('No identifier provided');
		if (!$this->getSecret()) throw new Exception('No secret key provided');
	}

	/**
	 * @param $function
	 * @param array $params
	 *
	 * @return array
	 * @throws Exception
	 */
	public function exec($function, $params=[]) {
		if (!$function) throw new Exception('you must provide function');
		$this->_validate();

		$params['identifier'] = $this->getIdentifier();
		$params['secret'] = $this->getSecret();
		$params['responsetype'] = 'json';
		$params['action'] = $function;

		$args = http_build_query($params, '', '&');
		$url =  $this->getProtocol() . '://' . $this->getURL() . '/includes/api.php';

		$response='';
		switch($this->getHTTPClient()) {
			case self::HTTP_CLIENT_CURL: $response = $this->_curl($url, $args); break;
			case self::HTTP_CLIENT_FOPEN: $response = $this->_fopen($url, $args); break;
		}

		if (stristr($response, '<html') == true) {
			if (stristr($response, 'Login Attempt Failed') == true) {
				throw new Exception("Login Attempt Failed");
			}
			if (stristr($response, 'action="/login/"') == true) {
				throw new Exception("Authentication Error");
			}
			throw new Exception("Invalid response data");
		}

		$response = json_decode($response, true);
		if($response === false) throw new Exception("Invalid response data");

		return $response;
	}

	/**
	 * @param $url
	 * @param $params
	 *
	 * @return mixed
	 * @throws Exception
	 */
	private function _curl($url, $params)
	{
		$curl = curl_init();
		curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, 0);
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 0);
		curl_setopt($curl, CURLOPT_URL, $url);
		curl_setopt($curl, CURLOPT_BUFFERSIZE, 131072);
		$header[0] =
			"Content-Type: application/x-www-form-urlencoded\r\n" .
			"Content-Length: " . strlen($params) . "\r\n" . "\r\n" . $params;
		curl_setopt($curl, CURLOPT_HTTPHEADER, $header);
		curl_setopt($curl, CURLOPT_POST, 1);
		$result = curl_exec($curl);
		if ($result == false) throw new Exception("curl_exec threw error \"" . curl_error($curl) . "\" for " . $url . "?" . $params );
		curl_close($curl);
		return $result;
	}

	/**
	 * @param $url
	 * @param $params
	 *
	 * @return bool|string
	 * @throws Exception
	 */
	private function _fopen($url, $params)
	{
		if (!(ini_get('allow_url_fopen'))) throw new Exception('fopen called on system without allow_url_fopen enabled in php.ini');

		$opts = array(
			'http' => array(
				'allow_self_signed' => true,
				'method' => 'POST',
				'header' =>
					"Content-Type: application/x-www-form-urlencoded\r\n" .
					"Content-Length: " . strlen($params) . "\r\n" .
					"\r\n" . $params
			)
		);
		$context = stream_context_create($opts);
		return file_get_contents($url, false, $context);
	}
}